//+------------------------------------------------------------------+
//|                                                  VininI_nEMA.mq4 |
//|                     Copyright, 2010 Victor I. Nicolaev aka Vinin |
//|                                             e-mail:vinin@mail.ru |
//+------------------------------------------------------------------+
#property copyright "Copyright, 2010 Victor I. Nicolaev"
#property link      "e-mail_to: vinin@mail.ru"

#property indicator_chart_window
#property indicator_buffers 1
#property indicator_color1 Red

//---- input parameters
extern int       EMA_period=89;     //  EMA
extern int price_mode=0;            //   0 - PRICE_CLOSE, 
                                    //                   1 - PRICE_OPEN,
                                    //                   2 - PRICE_HIGH,
                                    //                   3 - PRICE_LOW,
                                    //                   4 - PRICE_MEDIAN,
                                    //                   5 - PRICE_TYPICAL,
                                    //                   6 - PRICE_WEIGHTED
extern int mode_ema=0;              //   ,  6

//---- buffers
double Ema[];

double K;
int PrevTime;
double nEMA[][2];
double KEMA[];

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
   
   int i;
   SetIndexStyle(0,DRAW_LINE);
   SetIndexBuffer(0,Ema);

   SetIndexDrawBegin(0,EMA_period*(mode_ema+1));
   IndicatorShortName(mode_ema+"-EMA("+EMA_period+")");

   K=2.0/(EMA_period+1);
   PrevTime=0;   
   if (mode_ema>12) mode_ema=12;
   if (mode_ema<0)  mode_ema=0;
   ArrayResize(nEMA, mode_ema+1);
   ArrayResize(KEMA, mode_ema+2);
   KEMA[0]=1;
   for (i=1;i<=mode_ema+1;i++) {
      KEMA[i]=C(i, mode_ema+1);
      if (i%2==0) KEMA[i]*=-1;
   }
      
   
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
  {
//----
   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
  {
   int   i,j,
         limit,
         counted_bars=IndicatorCounted();
//----
   limit=Bars-counted_bars-1;
   if (counted_bars==0) {
      limit=Bars-1;
   }
   for (i=limit;i>=0;i--) {
      if (NevBar(i)) for (j=0;j<=mode_ema;j++) nEMA[j][1]=nEMA[j][0];
      nEMA[0][0]=nEMA[0][1]*(1.0-K)+fPrice(price_mode,i)*K;
      double Res=nEMA[0][0]*KEMA[1];
      for (j=1;j<=mode_ema;j++) {
         nEMA[j][0]=nEMA[j][1]*(1.0-K)+nEMA[j-1][0]*K;
         Res+=nEMA[j][0]*KEMA[j+1];
      }
      Ema[i]=Res;
   }   

   return(0);
  }
//+------------------------------------------------------------------+

double fPrice(int mode, int pos){
   switch (mode) {
      case PRICE_CLOSE:    return(Close[pos]);
      case PRICE_OPEN:     return(Open[pos]);
      case PRICE_HIGH:     return(High[pos]); 
      case PRICE_LOW:      return(Low[pos]); 
      case PRICE_MEDIAN:   return((High[pos]+Low[pos])/2.0);
      case PRICE_TYPICAL:  return((High[pos]+Low[pos]+Close[pos])/3.0);
      case PRICE_WEIGHTED: return((High[pos]+Low[pos]+Close[pos]*2.0)/4.0); 
   }
}

int C(int K, int N){
   int Res=FACT(N)/(FACT(K)*FACT(N-K));
   return(Res);
}

bool NevBar(int pos){
   if (PrevTime==Time[pos]) return(false);
   PrevTime=Time[pos];
   return(true);
}

int FACT(int N){
   int Res=1;
   for (int i=2;i<=N;i++)
      Res*=i;
   return(Res);
}